MESSAGE_KEY_CLIENT_ATTRIBUTE_NAME = 'messageKey'
GROUP_KEY_CLIENT_ATTRIBUTE_NAME = 'groupKey'
MESSAGE_HEADERS_KEY_CLIENT_ATTRIBUTE_NAME = 'messageHeaders'

MESSAGE_KEY_KEY = 'key'
MESSAGE_QUEUE_KEY_KEY = 'queueKey'
MESSAGE_GROUP_KEY = 'groupKey'
MESSAGE_CONTENT_KEY = 'content'
